#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const fs_1 = require("mz/fs");
const path_1 = require("path");
const cli_color_1 = require("cli-color");
const fileList_1 = require("./fileList");
const jsonDiff = require('json-diff');
main(minimist(process.argv.slice(2), {
    boolean: ['file'],
    alias: {
        help: ['h'],
        output: ['o'],
        file: ['f']
    }
}));
async function main(argv) {
    if (argv.help) {
        printHelp();
        process.exit(0);
    }
    const output = argv.output;
    if (argv.file) {
        const left = argv._[0];
        const right = argv._[1];
        const result = {
            baseDir: { left, right, },
            jsonFileDiff: {},
        };
        try {
            const leftJson = JSON.parse((await fs_1.readFile(left)).toString());
            const rightJson = JSON.parse((await fs_1.readFile(right)).toString());
            const diffString = jsonDiff.diff(leftJson, rightJson);
            result.jsonFileDiff[left] = diffString;
        }
        catch (error) {
            result.jsonFileDiff[left] = error.message;
        }
        await writeOutput(JSON.stringify(result, undefined, 2), output);
        return;
    }
    try {
        const left = await fileList_1.generateFileList(argv._[0]);
        const right = await fileList_1.generateFileList(argv._[1]);
        const fileDiff = fileList_1.generateFileListDiff(left, right);
        const result = Object.assign({}, fileDiff, { jsonFileDiff: {} });
        for (const file of fileDiff.all) {
            try {
                const leftJson = JSON.parse((await fs_1.readFile(path_1.join(fileDiff.baseDir.left, file))).toString());
                const rightJson = JSON.parse((await fs_1.readFile(path_1.join(fileDiff.baseDir.right, file))).toString());
                const diffString = jsonDiff.diff(leftJson, rightJson);
                result.jsonFileDiff[file] = diffString;
            }
            catch (error) {
                result.jsonFileDiff[file] = error.message;
            }
        }
        await writeOutput(JSON.stringify(result, undefined, 2), output);
    }
    catch (e) {
        console.error(cli_color_1.whiteBright.bgRedBright('error'), e);
        process.exit(1);
    }
}
function writeOutput(ts, argOut) {
    if (!argOut) {
        try {
            process.stdout.write(ts);
            return Promise.resolve();
        }
        catch (err) {
            return Promise.reject(err);
        }
    }
    return fs_1.writeFile(argOut, ts);
}
function printHelp() {
    const pkg = require('../package.json');
    process.stdout.write(`
${pkg.name} ${pkg.version}
Usage:
  jsondiff-report [--output, -o] [OUT_FILE] dir1 dir2
  jsondiff-report [--file, -f] [--output, -o] [OUT_FILE] file1 file2
  jsondiff-report [--help, -h]
Generate a JSON-diff report for 2 directories/files.

positional arguments:
  dir1/file1
  dir2/file2

optional arguments:
  -h, --help    show this help message and exit
  -f, --file    compare 2 files
  -o, --output  the report output file, if this option is absent,
                the programe will use the standard output

Output Format:
| Property       | Description                                                           |
| -------------- | --------------------------------------------------------------------- |
| baseDir        | The base dir: left for dir1 / file1, right for dir2 / file2.          |
| leftOnly       | List of files that only exists in baseDir.left.                       |
| rightOnly      | List of files that only exists in baseDir.right.                      |
| all            | List of files that exists in both baseDir.left and baseDir.right.     |
| jsonFileDiff   | The json-diff result for every file                                   |
`);
}
//# sourceMappingURL=cli.js.map